package ru.ifmo.ga.ca.configuration;

/**
 * User: Yuri D. Bedny
 * Date: Apr 14, 2006
 * Time: 4:31:41 PM
 */

public class ConfigurationFactory implements Factory {

    public static enum Implementation {
        BitSet,
        BooleanArray
    }

    private final Implementation impl;

    public ConfigurationFactory(Implementation impl) {
        this.impl = impl;
    }

    public Configuration create(int size, long x) {
        switch(impl) {
            case BitSet:
                return new BitSetConfiguration(size, x);
            case BooleanArray:
                return new BooleanArrayConfiguration(size, x);

            default:
                throw new IllegalArgumentException();
        }
    }

    public Configuration create(String s) throws IllegalArgumentException {
        switch(impl) {
            case BitSet:
                return new BitSetConfiguration(s);
            case BooleanArray:
                return new BooleanArrayConfiguration(s);

            default:
                throw new IllegalArgumentException();
        }
    }

    public Configuration create(int size) {
        switch(impl) {
            case BitSet:
                return new BitSetConfiguration(size);
            case BooleanArray:
                return new BooleanArrayConfiguration(size);

            default:
                throw new IllegalArgumentException();
        }

    }

    public Configuration create(boolean[] f) {
        switch(impl) {
            case BitSet:
                return new BitSetConfiguration(f);
            case BooleanArray:
                return new BooleanArrayConfiguration(f);

            default:
                throw new IllegalArgumentException();
        }
    }
}
