package ru.ifmo.ga.ca.dct.modules;

import ru.ifmo.ga.fx.Point;
import ru.ifmo.ga.fx.stat.Divergence;
import ru.ifmo.ga.ca.dct.CAModule;
import ru.ifmo.ga.ca.dct.CAPoint;
import ru.ifmo.ga.ca.Utils;

import java.util.TreeSet;
import java.util.ArrayList;
import java.util.Collections;

public class ExtremumSearcherModule extends CAModule {

    private static final int DIST = 32;
    private final Divergence div;
    private static final double MIN_VALUE = 0.80;
    private static final int UNIQUE_DIST = 4;

    public String getName() {
        return "ExtremumSearcherModule";
    }

    public ExtremumSearcherModule() throws Exception {
        ArrayList<CAPoint> all = new ArrayList<CAPoint>();
        div = repository.getDivergence();
        ArrayList<ArrayList<Point>> lines = new ArrayList<ArrayList<Point>>();
        TreeSet<Point> points = repository.getPoints();
//        System.out.println(points.size());


        double maxv = 0;
        String maxRule = "";
        int cnt = 0;
        ArrayList<Point> minLine = null;
        for (Point p : points) {
            int ds = Utils.diffFromKnown(((CAPoint)p).getRule().toString());
            if (ds >= UNIQUE_DIST) {
//                System.out.println(p.getMetadata().getValue());
                if (p.getMetadata().getValue() > maxv) {
                    maxv = p.getMetadata().getValue();
                    maxRule =((CAPoint)p).getRule().toString();
                }
            }


            if (p.getMetadata().getValue() < MIN_VALUE) {
                continue;
            }

            cnt++;
            all.add((CAPoint) p);
            // get min distance
            int min = Integer.MAX_VALUE;
            for (ArrayList<Point> line : lines) {
                int d = dist(line, p);
                if (d < min) {
                    min = d;
                    minLine = line;
                }
            }

            if (minLine == null || min > DIST) {
                ArrayList<Point> l = new ArrayList<Point>();
                lines.add(l);
                l.add(p);
            } else {
                minLine.add(p);
            }
        }

        int n = all.size();
        double diverg = 0;
        for (int i = 0; i < n - 1; i++) {
            for (int j = i + 1; j < n; j++) {
                diverg += div.getDivergence(all.get(i), all.get(j));
            }
        }
        diverg /= (n * (n - 1) / 2);

        System.out.println("Number of lines: " + lines.size() + ", number of points: " + cnt + ", div: " + diverg);
        for (ArrayList<Point> line : lines) {
            System.out.println("-------------------------------------------");
            Collections.sort(line);
            for (Point p : line) {
                CAPoint ca = (CAPoint) p;
                System.out.println(ca.getMetadata().getValue() + "\t" + ca.getRule().toString());
            }
        }

        System.out.println("dist: " + UNIQUE_DIST + ", value: " + maxv);
        System.out.println(maxRule);
    }

    private int dist(ArrayList<Point> line, Point p) {
        int max = 0;
        for (Point point : line) {
            int d = (int) div.getDivergence(point, p);
            if (d > max) {
                max = d;
            }
        }
        return max;
    }

    public static void main(String[] args) throws Exception {
        new ExtremumSearcherModule();
    }

}
