package ru.ifmo.ga.ca.dct.modules;

import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;
import ru.ifmo.ga.ca.dct.CAPoint;
import ru.ifmo.ga.ca.dct.CAModule;
import ru.ifmo.ga.ca.dct.ICStorehouse;
import ru.ifmo.ga.ca.randomizer.Randomizer;
import ru.ifmo.ga.ca.randomizer.RandomizerFactory;
import ru.ifmo.ga.ca.tournament.*;
import ru.ifmo.ga.ca.tree.Expression;
import ru.ifmo.ga.ca.tree.Tree;
import ru.ifmo.ga.fx.Point;
import ru.ifmo.ga.fx.desire.PointsInDeltaDesire;

import java.util.Arrays;
import java.util.Date;
import java.util.Hashtable;
import java.util.ArrayList;
import java.rmi.RemoteException;

/**
 * User: Yuri D. Bedny
 * Date: Apr 21, 2006
 * Time: 9:22:33 AM
 */

public class GP extends CAModule {

    private Tree downloadedTree;
    private CAPoint uploadedCAPoint = new CAPoint();
    private Rule uploadedRule = new Rule();

    public String getName() {
        return "GP";
    }


    // ------------------- COMMON -------------

    static final int FIELD_SIZE = 149;
    private static final int GENERATIONS = 10000;
    private static final int CA_POPULATION_SIZE = 100;
    private static final int IC_POPULATION_SIZE = 100;

    // ------------------- IC ------------------
    private final boolean USING_E = true;

    private final double [] ic_fintess = new double[IC_POPULATION_SIZE];
    private final int [] ic_rawFintess = new int[IC_POPULATION_SIZE];
    private ICOrder[] ic_order = new ICOrder[IC_POPULATION_SIZE];
    private final Configuration[] ic_population = new Configuration[IC_POPULATION_SIZE];
    private int ic_maxRaw;

    private Randomizer rndOverDensity = new RandomizerFactory(RandomizerFactory.Imlementation.OverDensity, ConfigurationFactory.Implementation.BooleanArray).create(FIELD_SIZE);
    private Randomizer rndEveryBit = new RandomizerFactory(RandomizerFactory.Imlementation.EveryBit, ConfigurationFactory.Implementation.BooleanArray).create(FIELD_SIZE);
    private static final double IC_BEST_KEEP_RATE = 0.95;
    private static final int IC_NUM_BEST_KEEP = (int) (IC_POPULATION_SIZE * IC_BEST_KEEP_RATE);

    private void fromRepository(Tree[] arr, int size, double from, double to) {
        try {
            ArrayList<Point> points = repository.downloadPoints(new PointsInDeltaDesire(from, to, getName(), size));
            System.out.println("Points are downloaded. Number: " + points.size());
            for (int i = 0; i < points.size(); i++) {
                CAPoint p = (CAPoint) points.get(i);
                arr[CA_POPULATION_SIZE - i - 1] = new Tree(p.getRule(), VARS);
            }
        } catch (RemoteException e) {
            System.out.println("Can't download points from repository");
            System.out.println(e);
        }
    }

    private void init_ic() {
        for (int i = 0; i < IC_POPULATION_SIZE; i++) {
            ic_population[i] = Math.random() > 0.5 ? rndOverDensity.random() : rndEveryBit.random();
        }

        ICStorehouse.set(ic_population);

        for (int i = 0; i < IC_POPULATION_SIZE; i++) {
            ic_order[i] = new ICOrder(i);
        }

    }

    private double e(CAContext ca, int d) {//todo: simple impl
        if (!USING_E) {
            return 1.0;
        }
        double all = 0;
        double lost = 0;
        for (ICContext c : ca.getLost()) {
            if (c.getCfg().cardinality() == d) {
                lost++;
                all++;
            }
        }

        for (ICContext c : ca.getWin()) {
            if (c.getCfg().cardinality() == d) {
                all++;
            }
        }

        double p = lost / all;
        double q = 1 - p;
        return (q == 1 || q == 0) ? Math.log(2.0) : Math.log(2.0) + p * Math.log(p) + q * Math.log(q);
    }

    private void calc_ic(Results results) {
        ic_avgDensity = 0;

        for (int i = 0; i < IC_POPULATION_SIZE; i++) {
            ICContext ic = results.getIcCtxs().get(i);
            ic_avgDensity += Math.abs(ic.getCfg().cardinality() - 1.0 * ic.getCfg().size() / 2);
        }
        ic_avgDensity /= IC_POPULATION_SIZE;

        ic_avgRawFintess = 0;
        ic_maxRaw = 0;
        for (int i = 0; i < IC_POPULATION_SIZE; i++) {
            double s = 0;
            ICContext ic = results.getIcCtxs().get(i);
            ic_rawFintess[i] = ic.getWin().size();

            ic_avgRawFintess += ic_rawFintess[i];
            if (ic_rawFintess[i] > ic_maxRaw) {
                ic_maxRaw = ic_rawFintess[i];
            }

            for (CAContext ca : ic.getWin()) {
                double t = 0;
                for (ICContext l : ca.getLost()) {
                    t += e(ca, l.getCfg().cardinality());
                }
                s += e(ca, ic.getCfg().cardinality()) / t;
            }
            ic_fintess[i] = ic_rawFintess[i] + s;
        }
        ic_avgRawFintess /= IC_POPULATION_SIZE;
        Arrays.sort(ic_order);
    }

    private void next_ic() {
        Configuration[] temp = new Configuration[IC_POPULATION_SIZE];
        for (int i = 0; i < IC_POPULATION_SIZE; i++) {
            temp[i] = (i < IC_NUM_BEST_KEEP) ? ic_population[ic_order[i].ind] : (Math.random() > 0.5 ? rndOverDensity.random() : rndEveryBit.random());
        }
        System.arraycopy(temp, 0, ic_population, 0, IC_POPULATION_SIZE);
        ICStorehouse.set(ic_population);
    }

    // ------------------- STATISTICS --------------
    int[] monotony = new int[CA_POPULATION_SIZE];
    int[] density = new int[CA_POPULATION_SIZE];
    double avgMonotony;
    double avgDiversity;

    double avgDensity;
    double ic_avgDensity;


    double ic_avgRawFintess;
    double avgRawFintess;

    double evolution;
    int lastWin = 1;
    double lastICavgDensity = FIELD_SIZE / 2;
    // ------------------- CAPoint ----------------------

    private static final boolean ELITISM_SELECTION = false;

    static final int RULE_RADIUS = 3;
    private static final int GENE_LENGTH = (1 << (2 * RULE_RADIUS + 1));

    private static final double RECOMBINATION_RATE = 0.7;
    private static final double MUTATION_RATE = 0.1;

    private static final double CA_BEST_KEEP_RATE = 0.2;
    private static final int NUM_BEST_KEEP = (int) (CA_POPULATION_SIZE * CA_BEST_KEEP_RATE);
    private static final double CA_GENERATE_RANDOM_RATE = 0.05;
    private static final int NUM_GENERATE_RANDOM = (int) (CA_POPULATION_SIZE * CA_GENERATE_RANDOM_RATE);

    private static final int STEPS = 2 * FIELD_SIZE;

    private final Tree [] population = new Tree[CA_POPULATION_SIZE];
    private final double [] fitness = new double[CA_POPULATION_SIZE];
    private final int [] rawFintess = new int[CA_POPULATION_SIZE];
    private final Order[] order = new Order[CA_POPULATION_SIZE];
    private double sumFitness;
    private int best = 0;

    private Hashtable<String, Integer> goodRules = new Hashtable<String, Integer>();
    private final Tournament trn = new Tournament();
    private final Tournament t = new Tournament();


    private static final int TEST_CASES = 10000;

    private class Order implements Comparable<Order>{
        public final int ind;

        public Order(int i) {
            this.ind = i;
        }

        public int compareTo(Order o) {
            if (fitness[ind] > fitness[o.ind])
                return -1;
            else if (fitness[ind] < fitness[o.ind])
                return 1;
            else
                return 0;
        }
    }

    private class ICOrder implements Comparable<ICOrder>{
        public final int ind;

        public ICOrder(int i) {
            this.ind = i;
        }

        public int compareTo(ICOrder o) {
            if (ic_fintess[ind] > ic_fintess[o.ind])
                return -1;
            else if (ic_fintess[ind] < ic_fintess[o.ind])
                return 1;
            else
                return 0;
        }
    }

    private GP() {
        System.out.println(new Date().toString());

        System.out.println("-----------------------------------------------------------");
        System.out.println("GP on trees");
        System.out.println("ca population size: " + CA_POPULATION_SIZE);
        System.out.println("ic population size: " + IC_POPULATION_SIZE);
        System.out.println("gene length: " + GENE_LENGTH);
        System.out.println("test cases: " + TEST_CASES);
        System.out.println("elitism selection: " + (ELITISM_SELECTION ? "yes" : "no"));
        System.out.println("mutation rate: " + MUTATION_RATE);
        System.out.println("recombination rate: " + RECOMBINATION_RATE);
        System.out.println("ca best keep rate: " + CA_BEST_KEEP_RATE);
        System.out.println("ca generate random rate: " + CA_GENERATE_RANDOM_RATE);
        System.out.println("ic best keep rate: " + IC_BEST_KEEP_RATE);
        System.out.println("using E: " + (USING_E ? "yes" : "no"));
        System.out.println("generations: " + GENERATIONS);
        System.out.println("-----------------------------------------------------------");

        init();
        init_ic();
        randomize();
        fromRepository(population, CA_POPULATION_SIZE, 0.7, 0.9);

        run(GENERATIONS);
    }

    private void run(int generations) {
        Tree [] next = new Tree[CA_POPULATION_SIZE];
        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            next[i] = new Tree(false, VARS);
        }

        for (int round = 0; round < generations; round++) {

            calc();

            final String SP = "\t\t\t\t\t";
            String ca_rf = rawFintess[order[0].ind] == maxRaw ? "" : "x";
            String ic_rf = ic_rawFintess[ic_order[0].ind] == ic_maxRaw ? "" : "x";
            double icd = Math.abs(ic_population[ic_order[0].ind].cardinality() - 1.0 * ic_population[ic_order[0].ind].size() / 2);
            System.out.println(
                            round + ".\t" +
                            avgRawFintess +  " (" + rawFintess[order[0].ind] % IC_POPULATION_SIZE + ca_rf + ")" + SP +
                            ic_avgRawFintess + " (" + ic_rawFintess[ic_order[0].ind] % CA_POPULATION_SIZE + ic_rf + ")"  + SP +
                            avgDensity + " (" + density[order[0].ind] + ")" + SP +
                            ic_avgDensity + " (" + icd + ")" + SP +
                            avgMonotony + " (" + monotony[order[0].ind] + ")" + SP +
                            (int)avgDiversity
            );

            if (round > 0 && round % 10 == 0) {
                stat();
            }

            int h = 0;
            for (int b = 0; b < NUM_BEST_KEEP; b++) {
                while (h < CA_POPULATION_SIZE && exists(population[order[h].ind], next, b)) {
                    h++;
                }
                if (h < CA_POPULATION_SIZE) {
                    GPUtils.copy(population[order[h].ind], next[b]);
                }
            }

            for (int i = NUM_BEST_KEEP; i < CA_POPULATION_SIZE - NUM_GENERATE_RANDOM; i++) {
                evolve(next[i]);
            }

            // download some rules from repository            
            if (round > 0 && round % 10 == 0) {
                    fromRepository(next, NUM_GENERATE_RANDOM, 0.7, 0.9);
            } else {
                for (int i = CA_POPULATION_SIZE - NUM_GENERATE_RANDOM; i < CA_POPULATION_SIZE; i++) {
                    randomize(next[i]);
                }
            }

//            if (downloadedTree != null) {
//                GPUtils.copy(downloadedTree, next[CA_POPULATION_SIZE - 1]);// copy to the last position
//                downloadedTree = null;// used now
//            }

            for (int i = 0; i < CA_POPULATION_SIZE; i++) {
                GPUtils.copy(next[i], population[i]);
            }

            next_ic();
        }
        stat();
    }

    private boolean exists(Tree chr, Tree[] next, int b) {
        for (int i = 0; i < b; i++) {
            if (GPUtils.equals(chr, next[i])) {
                return true;
            }
        }
        return false;
    }

    private void randomize() {
        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            population[i] = new Tree(false, VARS);
            randomize(population[i]);
        }
        try {
            population[0].setExpression(new Expression(Utils.getRule("00000001001010100000010110100001011000010011011110001000110111010000001110001111000000011000000110001000111111000101111100101111"), VARS));
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
/*
        try {
            population[0].setExpression(new Expression(Utils.getKnownRule(KnownCARepository.JP_2), VARS));
            population[1].setExpression(new Expression(Utils.getKnownRule(KnownCARepository.JP_1), VARS));
            population[2].setExpression(new Expression(Utils.getKnownRule(KnownCARepository.GKL), VARS));
            population[4].setExpression(new Expression(Utils.getKnownRule(KnownCARepository.GEP_1), VARS));
            population[5].setExpression(new Expression(Utils.getKnownRule(KnownCARepository.GEP_2), VARS));
            population[6].setExpression(new Expression(Utils.getKnownRule(KnownCARepository.KOZA), VARS));
        } catch (Exception e) {

        }
*/
    }


    private void init() {
        t.getSettings().setSize(FIELD_SIZE);
        t.getSettings().setRuns(IC_POPULATION_SIZE);
        t.getSettings().setSteps(STEPS);
        t.getSettings().setRandomizerImpl(RandomizerFactory.Imlementation.CA);

        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            Rule r = new Rule();
            r.setRadius(RULE_RADIUS);
            t.getRules().add(r);
        }

        trn.getSettings().setSize(FIELD_SIZE);
        trn.getSettings().setRuns(TEST_CASES);
        trn.getSettings().setSteps(STEPS);
        Rule r = new Rule();
        r.setRadius(RULE_RADIUS);
        trn.getRules().add(r);

        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            order[i] = new Order(i);
        }

        randomRule = new Rule();
        randomRule.setRadius(3);

        uploadedRule.setRadius(RULE_RADIUS);
        uploadedCAPoint.setRule(uploadedRule);
    }

    private String perc(int win) {
        return win * 100.0 / TEST_CASES + "%";
    }

    private void stat() {
        Rule r = trn.getRules().get(0);
        GPUtils.getRule(r, population[order[0].ind]);
        Integer win = goodRules.get(r.toString());
        boolean was = (win != null);
        if (!was) {
            Result res = trn.play().getResults().get(0);
            goodRules.put(r.toString(), res.getWin());
            win = res.getWin();
            upload(r, TEST_CASES, res.getValue());
        }

        if (best < win) {
            best = win;
        }
        System.out.println();
        System.out.println((was ? "[!] " : "") + "best: " + perc(win) + " (" + perc(best) + ")");
        System.out.println(rules[order[0].ind]);

        // for evolution statistics
        evolution = (win - lastWin) * 1.0 / lastWin +
                (lastICavgDensity - ic_avgDensity) / lastICavgDensity;


        System.out.println("evolution: " + (int)(evolution * 100) + "%");
        lastWin = win;
        lastICavgDensity = ic_avgDensity;

        // best rule from repository
//        Rule downloadedRule = download(rules[order[0].ind]);
//        downloadedTree = (downloadedRule != null) ? new Tree(downloadedRule, VARS) : null;
//        System.out.println();


    }

    private int select() {
        if (ELITISM_SELECTION) {
            return (int) (Math.random() * NUM_BEST_KEEP);
        }

        double x = Math.random() * sumFitness;
        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            int t = order[i].ind;
            x -= fitness[t];
            if (x < 0) {
                return t;
            }
        }
        return CA_POPULATION_SIZE - 1;
    }

    private void evolve(Tree p1) {
        final int i = select();

        GPUtils.copy(population[i], p1);

        if (Math.random() < RECOMBINATION_RATE) {
            crossover(p1);
        }

        if (Math.random() < MUTATION_RATE) {
            GPUtils.mutate(p1);
        }

    }


    private double maxRaw;

    private Rule[] rules = new Rule[CA_POPULATION_SIZE];

    private void calc() {
        avgDensity = 0;
        avgMonotony = 0;
        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            Rule r = t.getRules().get(i);
            GPUtils.getRule(r, population[i]);
            rules[i] = r;
            monotony[i] = r.getMonotony();
            density[i] = Math.abs(r.cardinality() - r.getLength() / 2);
            avgMonotony += monotony[i];
            avgDensity += density[i];
        }
        avgMonotony /= CA_POPULATION_SIZE;
        avgDensity /=  CA_POPULATION_SIZE;


        maxRaw = 0;
        avgRawFintess = 0;
        Results results = t.play();
        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            Result res = results.getResults().get(i);
            double tt = 0;
            for (ICContext ic : res.getWinCfgs()) {
                tt += 1.0 / ic.getLost().size();
            }
            fitness[i] = tt;

            rawFintess[i] = res.getWin();
            avgRawFintess += rawFintess[i];
            if (res.isBoth()) {
                if (rawFintess[i] * 4 < 3 * IC_POPULATION_SIZE) {
                    rawFintess[i] += IC_POPULATION_SIZE;
                } else if (rawFintess[i] * 4 >= 3 * IC_POPULATION_SIZE && rawFintess[i] * 20 < 17 * IC_POPULATION_SIZE) {
                    rawFintess[i] += 2 * IC_POPULATION_SIZE;
                } else if (rawFintess[i] * 20 >= 17 * IC_POPULATION_SIZE) {
                    rawFintess[i] += 3 * IC_POPULATION_SIZE;
                }
            }
            if (rawFintess[i] > maxRaw) {
                maxRaw = rawFintess[i];
            }
            fitness[i] += rawFintess[i] - density[i] / (avgDensity + 1) - monotony[i] / (avgMonotony + 1);
        }
        avgRawFintess /= CA_POPULATION_SIZE;
        Arrays.sort(order);

        avgDiversity = 0;
        final int m = CA_POPULATION_SIZE;
        for (int i = 0; i < m - 1; i++) {
            for (int j = i + 1; j < m; j++) {
                avgDiversity += div(rules[order[i].ind], rules[order[j].ind]);
            }
        }
        avgDiversity /= (m * (m - 1) / 2);

        sumFitness = 0;
        for (int i = 0; i < CA_POPULATION_SIZE; i++) {
            sumFitness += fitness[i];
        }

        calc_ic(results);
    }

    private int div(Rule r1, Rule r2) {
        int d = 0;
        for (int i = 0; i < r1.getLength(); i++) {
            if (r1.getBit(i) != r2.getBit(i)) {
                d++;
            }
        }
        return d;
    }

    public static void main(String[] args) {
        Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
        new GP();
    }

    // --------------------------------------------------------------------------------

    private Rule randomRule;
    public final static char[] VARS = new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'};




    public void randomize(Tree t) {
        for (int i = 0; i < GENE_LENGTH; i++) {
            randomRule.setBit(i,Math.random() > 0.5);
        }
        t.setExpression(new Expression(randomRule, VARS));
    }


    private void crossover(Tree t1) {
        if (t1.isTerminal()) {
            return;
        }

        Tree t2 = new Tree(false, VARS);
        int x = 0;
        do {
            int j = select();
            GPUtils.copy(population[j], t2);
            x++;
        } while (x < NUM_BEST_KEEP && GPUtils.equals(t1, t2));

        if (x == NUM_BEST_KEEP) {
            return;
        }

        if (t2.isTerminal()) {
            return;
        }

        // select non-terminal nodes
        Tree p1 = GPUtils.getRandomNode(t1);
        while (p1.isTerminal()) {
            p1 = GPUtils.getRandomNode(t1);
        }

        Tree p2 = GPUtils.getRandomNode(t2);
        while (p2.isTerminal()) {
            p2 = GPUtils.getRandomNode(t2);
        }

        // just swap node childs
        double d = Math.random();
        if (d < 0.25) {             // then - then
            Tree t = p1.getThen();
            p1.setThen(p2.getThen());
            p2.setThen(t);

        } else if (d < 0.5) {       // then - else
            Tree t = p1.getThen();
            p1.setThen(p2.getElse());
            p2.setElse(t);

        } else if (d < 0.75) {      // else - then
            Tree t = p1.getElse();
            p1.setElse(p2.getThen());
            p2.setThen(t);

        } else {                    // else - else
            Tree t = p1.getElse();
            p1.setElse(p2.getElse());
            p2.setElse(t);
        }

        // chose either t1 or t2 chromosome
        if (Math.random() < 0.5) {
            GPUtils.copy(t2, t1);
        }
    }



}
