package ru.ifmo.ga.ca.dct.modules;

import ru.ifmo.ga.ca.dct.CAModule;
import ru.ifmo.ga.ca.dct.CAPoint;
import ru.ifmo.ga.ca.dct.CAFitness;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.fx.Point;
import ru.ifmo.ga.fx.UploadResult;

import java.util.TreeSet;
import java.util.ArrayList;
import java.util.Collections;

public class RandomSearcherModule extends CAModule {
    final double MIN_VALUE = 0.8;
    final int NUM_BITS = 5;
    final int ATTEMPTS = 10000;
    private static final int TEST_CASES_1 = 100;
    private static final int TEST_CASES_2 = 5000;

    public RandomSearcherModule() throws Exception{
        CAFitness fitness = new CAFitness();
        System.out.println("min value: " + MIN_VALUE + ", num bits: " + NUM_BITS + ", attempts: " + ATTEMPTS + ", cases1: " + TEST_CASES_1 + ", cases2: " + TEST_CASES_2);
        ArrayList<Point> points = new ArrayList<Point>(repository.getPoints());
        Collections.reverse(points);
        int f[] = new int[NUM_BITS];

        for (Point point : points) {
            if (point.getMetadata().getValue() < MIN_VALUE) {
                continue;
            }

            CAPoint p  = (CAPoint) point;
            System.out.println("\t\ttrying to optimize... \t" + p.getMetadata().getValue());

            // get rule
            Rule r = p.getRule();

            // make an attempt with rule
            for (int a = 0; a < ATTEMPTS; a++) {
                System.out.println("attemp number: " + a);
                // randomize positions to flip
                for (int i = 0; i < NUM_BITS; i++) {
                    f[i] = (int) (Math.random() * r.getLength());
                }

                // flip
                for (int i = 0; i < NUM_BITS; i++) {
                    r.flipBit(f[i]);
                }

                // first quick check
                fitness.setTestCases(TEST_CASES_1);
                if (fitness.calculate(p) > MIN_VALUE) {

                    // final check
                    fitness.setTestCases(TEST_CASES_2);
                    double v = fitness.calculate(p);
                    if (v > MIN_VALUE) {
                        p.getMetadata().setTestCases(TEST_CASES_2);
                        p.getMetadata().setValue(v);
                        p.getMetadata().setClientName(getClass().getSimpleName());
                        if (repository.uploadPoint(p) == UploadResult.OK) {
                            System.out.println("\t\toptimized! " + v);
                        }
                    }

                }

                // flip back
                for (int i = 0; i < NUM_BITS; i++) {
                    r.flipBit(f[i]);
                }

            }
        }

    }

    public static void main(String[] args) throws Exception{
        Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
        new RandomSearcherModule();
    }
}
