package ru.ifmo.ga.ca.randomizer;

import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;

import java.util.ArrayList;

/**
 * User: Yuri D. Bedny
 * Date: Apr 8, 2006
 * Time: 8:46:48 PM
 */

class ConsequentRandomizer extends AbstractRandomizer {
    private long l;

    private boolean cached;
    private int ind;
    private ArrayList<Long> cache = new ArrayList<Long>();

    public ConsequentRandomizer(ConfigurationFactory.Implementation impl, int size) {
        super(impl, size);
    }

    private boolean exists(final long x) {
        long c = x;
        for (int i = 0; i < size - 1; i++) {
            c = Utils.cyclicShiftLeft(size, c);
            if (c < x)
                return true;
        }
        return false;
    }

    public Configuration random() {
        if (cached) {
            return ind < cache.size() ? factory.create(size, cache.get(ind++)) : null;
        }

        final long max = ((long)1 << size) - 1;
        while (l < max && exists(l)) {
            l++;
        }

        if (l >= max) {
            return null;
        }

        cache.add(l);
        return factory.create(size, l++);
    }

    public void init() {
        l = 0;
        ind = 0;
        cached = cache.size() > 0;
    }
}
