package ru.ifmo.ga.ca.randomizer;

import ru.ifmo.ga.ca.configuration.ConfigurationFactory;

/**
 * User: Yuri D. Bedny
 * Date: Apr 14, 2006
 * Time: 7:14:24 PM
 */

public class RandomizerFactory {
    private final Imlementation rndImpl;
    private final ConfigurationFactory.Implementation cfgImpl;

    public RandomizerFactory(Imlementation rndImpl, ConfigurationFactory.Implementation cfgImpl) {
        this.rndImpl = rndImpl;
        this.cfgImpl = cfgImpl;
    }

    public static enum Imlementation {
        Consequent,
        EveryBit,
        OverDensity,
        CA,
    }

    public Randomizer create(int size) {
        switch(rndImpl) {
            case EveryBit:
                return new EveryBitRandomizer(cfgImpl, size);
            case Consequent:
                return new ConsequentRandomizer(cfgImpl, size);
            case OverDensity:
                return new OverDensityRandomizer(cfgImpl, size);
            case CA:
                return new CARandomizer(cfgImpl, size);
            default:
                throw new IllegalArgumentException();
        }
    }
}
