package ru.ifmo.ga.ca.tournament;

import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.SimpleCA;
import ru.ifmo.ga.ca.configuration.Configuration;

import java.util.ArrayList;
import java.util.HashSet;

/**
 * User: Yuri D. Bedny
 * Date: Apr 30, 2006
 * Time: 11:44:55 AM
 */

public class CAContext {
    private final ArrayList<ICContext> win = new ArrayList<ICContext>();
    private final ArrayList<ICContext> lost = new ArrayList<ICContext>();
    private final Rule rule;
    private final Settings settings;
    private final SimpleCA ca;

    private final HashSet<Configuration> cachedWin = new HashSet<Configuration>();
    private final HashSet<Configuration> cachedLost = new HashSet<Configuration>();

    public SimpleCA getCa() {
        return ca;
    }

    public ArrayList<ICContext> getWin() {
        return win;
    }

    public ArrayList<ICContext> getLost() {
        return lost;
    }

    public Rule getRule() {
        return rule;
    }

    public CAContext(Rule rule, Settings settings) {
        this.rule = rule;
        this.settings = settings;

        ca = new SimpleCA();
        ca.setSize(settings.getSize());
        ca.setRule(rule);
        ca.setConfigurationImpl(settings.getConfigurationImpl());
    }


    public boolean play(ICContext ctx) {
        ca.setConfiguration(ctx.getCfg());

        ArrayList<Configuration> cached = new ArrayList<Configuration>();

        boolean ok = false;

        for (int step = 0; step < settings.getSteps(); step++) {

            // Check if automata already made an incorrect step. If so, break
            if (settings.isCorrectness()) {
                if (ca.getConfiguration().isOnesMajority() != ctx.getCfg().isOnesMajority()) {
                    ok = false;
                    break;
                }
            }

            // Check if this configuration was already analyzed, if so break
            if (settings.isCacheAnalyzed()) {
                if (cachedWin.contains(ca.getConfiguration()) || cachedLost.contains(ca.getConfiguration())) {
                    ok = cachedWin.contains(ca.getConfiguration());
                    break;
                }
                cached.add(ca.getConfiguration());
            }

            // Do next move and check if it was last, if so break
            if (!ca.next()) {
                ok = ca.getConfiguration().isClassified(ctx.getCfg().isOnesMajority());
                break;
            }

        }

        if (ok) {
            win.add(ctx);
        } else {
            lost.add(ctx);
        }

        // Cache analyzed configuration if needed
        if (settings.isCacheAnalyzed()) {
            if (ok) {
                cachedWin.addAll(cached);
            } else {
                cachedLost.addAll(cached);
            }
        }

        return ok;
    }

}
