package ru.ifmo.ga.ca.tournament;

import ru.ifmo.ga.ca.configuration.ConfigurationFactory;
import ru.ifmo.ga.ca.randomizer.Randomizer;
import ru.ifmo.ga.ca.randomizer.RandomizerFactory;

/**
 * User: Yuri D. Bedny
 * Date: Apr 9, 2006
 * Time: 10:43:44 AM
 */

public class Settings {
    private int size;
    private int steps;
    private boolean correctness;
    private long runs;
    private boolean cacheAnalyzed;

    private ConfigurationFactory.Implementation cfgImpl = ConfigurationFactory.Implementation.BooleanArray;
    private RandomizerFactory.Imlementation rndImpl = RandomizerFactory.Imlementation.EveryBit;

    private Randomizer randomizer;

    public Settings() {
        randomizer = new RandomizerFactory(rndImpl, cfgImpl).create(size);
    }

    public Settings(Settings settings) {
        size = settings.size;
        steps = settings.steps;
        correctness = settings.correctness;
        runs = settings.runs;
        cacheAnalyzed = settings.cacheAnalyzed;
        cfgImpl = settings.cfgImpl;
        rndImpl = settings.rndImpl;

        randomizer = new RandomizerFactory(rndImpl, cfgImpl).create(size);
    }

    public boolean isCacheAnalyzed() {
        return cacheAnalyzed;
    }

    public void setCacheAnalyzed(boolean cacheAnalyzed) {
        this.cacheAnalyzed = cacheAnalyzed;
    }

    public Randomizer getRandomizer() {
        return randomizer;
    }

    public ConfigurationFactory.Implementation getConfigurationImpl() {
        return cfgImpl;
    }

    private void updateRandomizer() {
        randomizer = new RandomizerFactory(rndImpl, cfgImpl).create(size);
    }

    public void setConfigurationImpl(ConfigurationFactory.Implementation cfgImpl) {
        this.cfgImpl = cfgImpl;
        updateRandomizer();
    }

    public RandomizerFactory.Imlementation getRandomizerImpl() {
        return rndImpl;
    }

    public void setRandomizerImpl(RandomizerFactory.Imlementation rndImpl) {
        this.rndImpl = rndImpl;
        updateRandomizer();
    }

    public boolean isCorrectness() {
        return correctness;
    }

    public void setCorrectness(boolean correctness) {
        this.correctness = correctness;
    }

    public int getSize() {
        return size;
    }

    public void setSize(int size) {
        this.size = size;
        updateRandomizer();
    }

    public long getRuns() {
        return runs;
    }

    public void setRuns(long runs) {
        this.runs = runs;
    }

    public int getSteps() {
        return steps;
    }

    public void setSteps(int steps) {
        this.steps = steps;
    }


    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("size: ").append(size).
                append(", steps: ").append(steps).
                append(", correctness: ").append(correctness).
                append(", runs: ").append(runs).
                append(", randomizer: ").append(randomizer.getClass().getSimpleName());
        return sb.toString();
    }
}
