package ru.ifmo.ga.ca.tree;

import ext.bmin.ProductTerm;
import ext.bmin.MinimizedTable;

import java.util.ArrayList;
import java.util.List;

import ru.ifmo.ga.ca.Rule;

/**
 * User: Yuri D. Bedny
 * Date: May 29, 2006
 * Time: 1:42:30 AM
 */

public class Expression {//immutable
    private final ArrayList<ProductTerm> terms;
    private final char[] vars;

    final boolean isTruth;
    final boolean isFalse;

    public Expression(Expression ex) {
        this(ex.getTerms(), ex.getVars());
    }

    public Expression(boolean isTruth, char[] vars) {
        terms = null;
        this.vars = new char[vars.length];
        System.arraycopy(vars, 0, this.vars, 0, vars.length);
        this.isTruth = isTruth;
        this.isFalse = !isTruth;
    }

    public Expression(Rule r, char[] vars) {
        this(new MinimizedTable(r.toCNF(vars), null));
    }

    public Expression(String s) {
        this(new MinimizedTable(s, null));
    }

    public boolean isTruth() {
        if (isTruth) {
            return true;
        }

        for (ProductTerm term : terms) {
            if (term.isTruth()) {
                return true;
            }
        }
        return false;
    }

    public boolean isFalse() {
        if (isFalse) {
            return true;
        }

        return terms.size() == 0;
    }

    public Expression(List<ProductTerm> terms, char[] vars) {
        this.isTruth = false;
        this.isFalse = false;
        this.terms = new ArrayList<ProductTerm>();
        for (ProductTerm term : terms) {
            this.terms.add(new ProductTerm(term));
        }

        this.vars = new char[vars.length];
        System.arraycopy(vars, 0, this.vars, 0, vars.length);
    }

    public Expression(MinimizedTable mt) {
        this(mt.getProductTerms(), mt.getVars());
    }

    public boolean calc(boolean[] values) {
        if (isTruth) {
            return true;
        }

        if (isFalse) {
            return false;
        }

        for (ProductTerm prod : terms) {
            if (prod.calc(values)) {
                return true;
            }
        }
        return false;
    }

    public char[] getVars() {
        return vars;
    }

    public ArrayList<ProductTerm> getTerms() {
        return terms;
    }

    public int getVarsNum() {
        return vars.length;
    }

    public Rule toRule() {
        Rule r = new Rule();
        final int d = vars.length;
        r.setRadius(d / 2);
        final int max = 1 << d;
        for (int i = 0; i < max; i++) {
            r.setBit(i, calc(getArr(d, i)));
        }
        return r;

    }

    private boolean[] getArr(int size, int x) {
        boolean[] a = new boolean[size];
        for (int i = 0; i < size; i++) {
            a[i] = (x & 1) == 1;
            x >>= 1;
        }
        return a;
    }

    public String toString() {
        if (isTruth) {
            return "1";
        }

        if (isFalse) {
            return "0";
        }

        StringBuilder sb = new StringBuilder();
        boolean first = true;
        for (ProductTerm term : terms) {
            if (!first) {
                sb.append(" + ");
            }
            sb.append(term.toString());
            first = false;
        }
        return sb.toString();
    }

}
