package ru.ifmo.ga.fx;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;
import java.util.Date;
import java.util.Hashtable;
import java.io.*;
import java.lang.reflect.InvocationTargetException;

//todo: all println's to logger

public abstract class AbstractServer implements Server {

    private final static String REP_LIST_FILE = "repositories";

    private RepositoryKeeper keeper = new RepositoryKeeper();
    private Hashtable<String, Repository> repositories = new Hashtable<String, Repository>();

    public Repository getRepository(String repositoryName) throws RemoteException {
        return repositories.get(repositoryName);
    }

    private void registerRepository(Repository rep) throws RemoteException {
        Remote repStub = UnicastRemoteObject.exportObject(rep, 0);
        LocateRegistry.getRegistry().rebind(rep.getName(), repStub);

        // put object to collection
        repositories.put(rep.getName(), (Repository) repStub);
    }

    public void createRepository(Repository repository) throws RemoteException {
        // check if repository with the same name already exists
        if (repositories.containsKey(repository.getName())) {
            throw new FrameworkException("Repository '" + repository.getName() + "' already exists.");
        }

        // create remote Repository object and register it
        registerRepository(repository);

        // add record to repository list file
        try {
            BufferedWriter bw = new BufferedWriter(new FileWriter(REP_LIST_FILE, true));
            bw.newLine();
            bw.write(repository.getName());
            bw.newLine();
            bw.write(repository.getClass().getName());
            bw.close();
        } catch (IOException e) {
            throw new FrameworkException(e);
        }

        // ok! created
        System.out.println("Repository '" + repository.getName() + "' created. Max size: " + repository.getMaxSize() + ".");
    }

    public void start() throws Exception {
        Remote serverStub = UnicastRemoteObject.exportObject(this, 0);
        for (String s : LocateRegistry.getRegistry().list()) {
            System.out.println(s);
        }
        LocateRegistry.getRegistry().rebind(getName(), serverStub);

        // load all repositories from woring directory
        try {
            loadRepositories();
        } catch (IOException e) {
            System.out.println("Can't load repositories from list file '" + REP_LIST_FILE + "'." + e.getMessage());
            e.printStackTrace();
        }

        // schedule task to backup all repositories
        keeper.schedule(repositories);

        // all is ok!
        System.out.println("[" + new Date().toString() + "] DCT server started.");
    }

    private void loadRepositories() throws IOException, ClassNotFoundException, IllegalAccessException, InstantiationException, NoSuchMethodException, InvocationTargetException {
        BufferedReader br = new BufferedReader(new FileReader(REP_LIST_FILE));
        String repName = br.readLine();
        while (repName != null) {
            String className = br.readLine();
            Class<Repository> clazz = (Class<Repository>) Class.forName(className);
            Repository rep = clazz.getConstructor(String.class).newInstance(repName);
            try {
                rep.load();
            } catch (IOException e) {
                System.out.println("Can't load repository '" + rep.getName() + "'. " + e.getMessage());
            }
            registerRepository(rep);
            System.out.println();
            repName = br.readLine();
        }
        br.close();
    }

    public void sendMessage(String s) {
        System.out.println(s);
    }


}
