package ru.ifmo.ga.fx.desire;

import ru.ifmo.ga.fx.Point;
import ru.ifmo.ga.fx.Repository;
import ru.ifmo.ga.fx.stat.Divergence;

import java.rmi.RemoteException;
import java.util.ArrayList;

public class PointsInDeltaDesire implements DownloadDesire{
    private final double minV;
    private final double maxV;
    private final String clientName;
    private final int size;

    public PointsInDeltaDesire(double minV, double maxV, String clientName, int size) {
        this.minV = minV;
        this.maxV = maxV;
        this.clientName = clientName;
        this.size = size;
    }

    public Point downloadPoint(Repository repository) throws RemoteException {
        return null;
    }

    public String getClientName() {
        return clientName;
    }

    public ArrayList<Point> downloadPoints(Repository rep) throws RemoteException {
        ArrayList<Point> points = new ArrayList<Point>();

        // check if no points are required
        if (size == 0) {
            return points;
        }

        // get all in range
        ArrayList<Point> from = new ArrayList<Point>();
        for (Point p : rep.getPoints()) {
            if (p.getMetadata().getValue() >= minV && p.getMetadata().getValue() <= maxV) {
                from.add(p);
            }
        }

        // no points in desired range
        if (from.size() == 0) {
            return points;
        }

        // add first
        points.add(from.get((int) (Math.random() * from.size())));

        // add others
        Divergence div = rep.getDivergence();
        int cnt = Math.min(size, from.size() - 1);
        for (int i = 1; i < cnt; i++) {
            Point best = null;
            int max = 0;
            for (Point p : from) {
                int d = dist(points, p, div);
                if (d > max) {
                    max = d;
                    best = p;
                }
            }
            points.add(best);
        }

        assert (points.size() <= size);
        return points;
    }

    private int dist(ArrayList<Point> points, Point p, Divergence div) {
        int min = Integer.MAX_VALUE;
        for (Point point : points) {
            int d = (int) div.getDivergence(point, p);
            if (d < min) {
                min = d;
            }
        }
        return min;
    }

}
