package ru.ifmo.ga.ca;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.tree.Tree;
import ru.ifmo.ga.ca.tree.Expression;

/**
 * User: Yuri D. Bedny
 * Date: May 30, 2006
 * Time: 2:11:08 AM
 */

public class JPTest extends TestCase {
    public void test() throws Exception {
        Rule jp = Utils.getRule(KnownCARepository.JP_2);
        for (int i = 0; i < jp.getLength(); i++) {
            jp.flipBit(i);
            double value = Utils.testRule(jp, 1000).getValue();
            System.out.println(i + "\t" + value);
            if (value > 0.81 && value < 0.82) {
                System.out.println(Utils.testRule(jp, 10000).getValue());
            }
            jp.flipBit(i);
            System.out.println();
        }
    }

    public void testFlip() throws Exception{
        Rule rule = Utils.getRule(KnownCARepository.JP_2);
        rule.flipBit(3);
        System.out.println(rule.getMonotony());
        Tree tree = new Tree(rule, new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        System.out.println(tree.toString());
        System.out.println(tree.size() + ", " + tree.height());
    }

    public void testFlip2() throws Exception {
        Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
        Rule jp = Utils.getRule("00000001000101000011000011010111000000010000111100111001010101110000111101110100111111110001011111010001001111011111100101010111");
//        int p[] = new int[]{81, 19, 104, 79};     //  82.6 (7)
//        int p[] = new int[]{70, 72, 98, 73};      //  0.84 (6)
//        int p[] = new int[]{98, 100, 60, 60};       // 0.832 (4)
//        int p[] = new int[]{20, 20, 17, 114};       // 0.835 (4)
//        int p[] = new int[]{19, 9, 71, 71};             // 0.837 (4)
//-------------------------------------------------------------------------------
//        int p[] = new int[]{79, 123, 18, 70, 123}; // 0.837    (7)
        int p[] = new int[]{72, 13, 66, 46, 120};
        for (int i = 0; i < p.length; i++) {
            jp.flipBit(p[i]);
        }
        System.out.println(jp.toString());
        System.out.println(Utils.diff(jp.toString(), KnownCARepository.JP_1));
        System.out.println(Utils.testRule(jp, 10000).getValue());
    }

    public void testRandomFlip() throws Exception {
/*

72
13
66
46
120



*/
        Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
        Rule jp = Utils.getRule("00000001000101000011000011010111000000010000111100111001010101110000111101110100111111110001011111010001001111011111100101010111");
        int p[] = new int[5];
        for (int i = 0; i < 100000; i++) {
            for (int j = 0; j < p.length; j++) {
                p[j] = (int) (Math.random() * jp.getLength());
                jp.flipBit(p[j]);
            }
            if (Utils.testRule(jp, 100).getValue() > 0.87) {
                for (int j : p) {
                    System.out.println(j + " ");
                }
                System.out.println();
            }
            for (int j : p) {
                jp.flipBit(j);
            }
        }
    }

    public void testOneFlip() throws Exception {
        Rule jp = Utils.getRule(KnownCARepository.JP_2);
        double max = 0;
        int maxi = -1;
        for (int i = 0; i < jp.getLength(); i++) {
            jp.flipBit(i);
            double value = Utils.testRule(jp, 1000).getValue();
            if (value > max) {
                max = value;
                maxi = i;
            }
            jp.flipBit(i);
        }
        System.out.println("max: " + max + ", maxi: " + maxi);
        jp.flipBit(maxi);
        System.out.println(Utils.testRule(jp, 10000).getValue());
    }

    public void testTwoFlips() throws Exception {
        Rule jp = Utils.getRule(KnownCARepository.JP_2);
/*
        double max = 0;
        int maxi = -1;
        int maxj = -1;
        for (int i = 0; i < jp.getLength() - 1; i++) {
            for (int j = i + 1; j < jp.getLength(); j++) {
                jp.flipBit(i);
                jp.flipBit(j);
                double value = Utils.testRule(jp, 1000).getValue();
                if (value > max) {
                    max = value;
                    maxi = i;
                    maxj = j;
                }
                jp.flipBit(i);
                jp.flipBit(j);
            }
        }
        System.out.println("max: " + max + ", maxi: " + maxi + ", maxj: " + maxj);
*/
        jp.flipBit(30);
        jp.flipBit(60);
        System.out.println(new Tree(jp, new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'}).height());
//        System.out.println(Utils.testRule(jp, 30000).getValue());
    }

    public void testFlipTreeComplexity() throws Exception{
        Rule jp = Utils.getRule(KnownCARepository.JP_2);
        int min = 1000;
        for (int i = 0; i < jp.getLength() - 1; i++) {
            for (int j = i + 1; j < jp.getLength(); j++) {
                jp.flipBit(i);
                jp.flipBit(j);
                final Expression expr = new Expression(jp, new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
                if (expr.getTerms().size() < min) {
                    min = expr.getTerms().size();
                    System.out.println(min);
                }
//                System.out.println(i + "\t" + expr.size() + '\t' + expr.height());
                jp.flipBit(i);
                jp.flipBit(j);
            }
        }
        System.out.println("min: " + min);
    }

}
