package ru.ifmo.ga.ca.configuration;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.configuration.Configuration;
import ru.ifmo.ga.ca.configuration.ConfigurationFactory;

import java.util.ArrayList;


/**
 * User: Yuri D. Bedny
 * Date: Apr 9, 2006
 * Time: 5:04:32 PM
 */

public class ConfigurationTest extends TestCase {

    private ArrayList<ConfigurationFactory> getFactories() {
        ArrayList<ConfigurationFactory> f = new ArrayList<ConfigurationFactory>();
        for (ConfigurationFactory.Implementation impl : ConfigurationFactory.Implementation.values())
            f.add(new ConfigurationFactory(impl));
        return f;
    }

    public void testEquals() {
        Configuration c1 = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create("001010111");
        Configuration c2 = new ConfigurationFactory(ConfigurationFactory.Implementation.BitSet).      create("001010111");
        Configuration c3 = new ConfigurationFactory(ConfigurationFactory.Implementation.BitSet).      create("001010110");
        Configuration shiftedC1ba = new ConfigurationFactory(ConfigurationFactory.Implementation.BooleanArray).create("100101011");
        Configuration shiftedC1bs = new ConfigurationFactory(ConfigurationFactory.Implementation.BitSet).create("100101011");

        assertEquals(c1, c2);
        assertFalse(c1.equals(c3));
        assertFalse(c2.equals(c3));

        assertTrue(c1.isCyclicEquals(shiftedC1ba));
        assertTrue(c1.isCyclicEquals(shiftedC1bs));
    }

    public void testFromToString() throws Exception {
        for (ConfigurationFactory f : getFactories()) {
            Configuration cfg = f.create("1100010101");
            assertEquals("1100010101", cfg.toString());

            for (int i = 0; i < (1 << 10); i++) {
                StringBuilder sb = new StringBuilder(Integer.toBinaryString(i));
                while (sb.length() < 10) {
                    sb.insert(0, '0');
                }
                Configuration c = f.create(sb.toString());
                assertEquals(sb.toString(), c.toString());
            }
        }
    }

    public void testFromToLong() {
        for (ConfigurationFactory f : getFactories()) {
            for (int i = 0; i < (1 << 10); i++) {
                Configuration c = f.create(10, i);
                assertEquals(i, c.toLong());
            }
        }
    }

    public void testFromBooleanArray() {
        for (ConfigurationFactory f : getFactories()) {
            boolean b[] = {false, false, true, false, true};
            Configuration c = f.create(b);
            assertEquals(c.toLong(), 20);
            assertEquals(c.toString(), "10100");
        }
    }
}
