package ru.ifmo.ga.ca.dct;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.tree.Tree;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.KnownCARepository;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.dct.modules.GEPUtils;
import ru.ifmo.ga.ca.dct.modules.GEP;

import java.lang.reflect.Field;

public class GEPTest extends TestCase {

    private boolean equals(int[] x, int []y) {
        int l = Math.min(x.length, y.length);
        for (int i = 0; i < l; i++) {
            if (x[i] != y[i]) {
                return false;
            }
        }
        return true;
    }

    private void checkRule(String desc, String s) throws Exception {
        final int HEAD_LENGTH = 30;
        Tree tree = new Tree(Utils.getRule(s), new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        assertEquals(tree.toRule().toString(), s);

        int ch[][] = new int[3][HEAD_LENGTH * 3 + 1];
        GEPUtils.treeToChromosome(tree, ch);
        Rule r = new Rule();
        r.setRadius(3);
        GEPUtils.express(r, ch, HEAD_LENGTH);
        assertEquals(desc, Utils.getRule(s), r);
    }

    public void testTreeToChromosome() throws Exception {
        //d(c(1, a(1, 0)), g(e(1, 0), 0))
        Tree tree = new Tree(Utils.getRule(KnownCARepository.GKL), new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        assertEquals(tree.toRule().toString(), KnownCARepository.GKL);

        int ch[][] = new int[3][61];
        GEPUtils.treeToChromosome(tree, ch);
        assertEquals(3, ch[1][0]);
        int[] ok = new int[]{-1, 2, GEP.TRUTH_TERM, -1, 0, GEP.TRUTH_TERM, GEP.FALSE_TERM};
        int[] bad = new int[] {-1, 6, -1, GEP.FALSE_TERM, 4, GEP.TRUTH_TERM, GEP.FALSE_TERM};
        assertTrue(equals(ok, ch[0]));
        assertTrue(equals(bad, ch[2]));
        Rule r = new Rule();
        r.setRadius(3);
        GEPUtils.express(r, ch, 20);
        assertEquals(Utils.getRule(KnownCARepository.GKL), r);


        Field[] fields = KnownCARepository.class.getFields();
        for (Field field : fields) {
            checkRule(field.getName(), (String)field.get(null));
        }

    }

}
