package ru.ifmo.ga.ca.tree;

import junit.framework.TestCase;
import ru.ifmo.ga.ca.Rule;
import ru.ifmo.ga.ca.Utils;
import ru.ifmo.ga.ca.KnownCARepository;

/**
 * User: Yuri D. Bedny
 * Date: May 29, 2006
 * Time: 10:44:55 PM
 */

public class TreeTest extends TestCase {
    public void testSimple() {
        Tree tree = new Tree("a");
        assertFalse(tree.isFalse());
        assertFalse(tree.isTruth());
        assertEquals(1, tree.getVars().length);

        assertFalse(tree.isTerminal());

        assertEquals(0, tree.getVarNum());
        assertTrue(tree.getThen().isTerminal());
        assertTrue(tree.getThen().isTruth());
        assertTrue(tree.getElse().isTerminal());
        assertTrue(tree.getElse().isFalse());
    }

    //              d'eg + cd + ad
    public void testGKLTree() throws Exception {
        Tree tree = new Tree(Utils.getRule(KnownCARepository.GKL), new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        assertFalse(tree.isTerminal());

        assertEquals('d', tree.getVarChar());// d
        assertEquals('c', tree.getThen().getVarChar()); // c
        assertTrue(tree.getThen().getThen().isTerminal()); // 1
        assertEquals('a', tree.getThen().getElse().getVarChar()); // 1
        assertTrue(tree.getThen().getElse().getThen().isTruth()); // 1
        assertTrue(tree.getThen().getElse().getElse().isFalse()); // 1

        assertEquals("d(c(1, a(1, 0)), g(e(1, 0), 0))", tree.toString());
        assertTrue(tree.calc(new boolean[]{true, false, false, true, false, false, false}));// da
        assertFalse(tree.calc(new boolean[]{false, false, false, true, false, false, false}));// d
        assertTrue(tree.calc(new boolean[]{false, false, true, true, false, false, false}));// dc
        assertTrue(tree.calc(new boolean[]{false, false, false, false, true, false, true}));// 'deg
        assertFalse(tree.calc(new boolean[]{false, false, false, true, true, false, true}));// deg
    }

    public void testJP2Tree() throws Exception {
        Tree tree = new Tree(Utils.getRule(KnownCARepository.JP_2), new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});
        System.out.println(tree.height());
        System.out.println(tree.size());
    }

    public void testToRule() throws Exception {
        Rule rule = Utils.getRule(KnownCARepository.JP_2);
        Tree tree = new Tree(rule, new char[]{'a', 'b', 'c', 'd', 'e', 'f', 'g'});

        assertEquals(rule, tree.toRule());
    }
}
